@@ -38,6 +38,7 @@
 #include "text-input-unstable-v1-server-protocol.h"
 #include "input-method-unstable-v1-server-protocol.h"
 #include "shared/helpers.h"
+#include "shared/timespec-util.h"
 
 struct text_input_manager;
 struct input_method;
@@ -64,7 +65,7 @@ struct text_input_manager {
 	struct wl_global *text_input_manager_global;
 	struct wl_listener destroy_listener;
 
-	struct text_input *current_panel;
+	struct text_input *current_text_input;
 
 	struct weston_compositor *ec;
 };
@@ -105,7 +106,7 @@ struct text_backend {
 		struct wl_client *client;
 
 		unsigned deathcount;
-		uint32_t deathstamp;
+		struct timespec deathstamp;
 	} input_method;
 
 	struct wl_listener client_listener;
@@ -132,6 +133,7 @@ deactivate_input_method(struct input_method *input_method)
 		zwp_input_method_v1_send_deactivate(
 			input_method->input_method_binding,
 			input_method->context->resource);
+		input_method->context->input = NULL;
 	}
 
 	wl_list_remove(&input_method->link);
@@ -139,11 +141,15 @@ deactivate_input_method(struct input_method *input_method)
 	input_method->context = NULL;
 
 	if (wl_list_empty(&text_input->input_methods) &&
-	    text_input->input_panel_visible) {
+	    text_input->input_panel_visible &&
+	    text_input->manager->current_text_input == text_input) {
 		wl_signal_emit(&ec->hide_input_panel_signal, ec);
 		text_input->input_panel_visible = false;
-		text_input->manager->current_panel = NULL;
 	}
+
+	if (text_input->manager->current_text_input == text_input)
+		text_input->manager->current_text_input = NULL;
+
 	zwp_text_input_v1_send_leave(text_input->resource);
 }
 
@@ -187,10 +193,14 @@ text_input_activate(struct wl_client *client,
 {
 	struct text_input *text_input = wl_resource_get_user_data(resource);
 	struct weston_seat *weston_seat = wl_resource_get_user_data(seat);
-	struct input_method *input_method = weston_seat->input_method;
+	struct input_method *input_method;
 	struct weston_compositor *ec = text_input->ec;
 	struct text_input *current;
 
+	if (!weston_seat)
+		return;
+
+	input_method = weston_seat->input_method;
 	if (input_method->input == text_input)
 		return;
 
@@ -205,12 +215,11 @@ text_input_activate(struct wl_client *client,
 
 	input_method_context_create(text_input, input_method);
 
-	current = text_input->manager->current_panel;
+	current = text_input->manager->current_text_input;
 
 	if (current && current != text_input) {
 		current->input_panel_visible = false;
 		wl_signal_emit(&ec->hide_input_panel_signal, ec);
-		text_input->manager->current_panel = NULL;
 	}
 
 	if (text_input->input_panel_visible) {
@@ -218,8 +227,8 @@ text_input_activate(struct wl_client *client,
 			       text_input->surface);
 		wl_signal_emit(&ec->update_input_panel_signal,
 			       &text_input->cursor_rectangle);
-		text_input->manager->current_panel = text_input;
 	}
+	text_input->manager->current_text_input = text_input;
 
 	zwp_text_input_v1_send_enter(text_input->resource,
 				     text_input->surface->resource);
@@ -232,7 +241,7 @@ text_input_deactivate(struct wl_client *client,
 {
 	struct weston_seat *weston_seat = wl_resource_get_user_data(seat);
 
-	if (weston_seat->input_method->input)
+	if (weston_seat && weston_seat->input_method->input)
 		deactivate_input_method(weston_seat->input_method);
 }
 
@@ -334,7 +343,8 @@ text_input_show_input_panel(struct wl_client *client,
 
 	text_input->input_panel_visible = true;
 
-	if (!wl_list_empty(&text_input->input_methods)) {
+	if (!wl_list_empty(&text_input->input_methods) &&
+	    text_input == text_input->manager->current_text_input) {
 		wl_signal_emit(&ec->show_input_panel_signal,
 			       text_input->surface);
 		wl_signal_emit(&ec->update_input_panel_signal,
@@ -352,10 +362,8 @@ text_input_hide_input_panel(struct wl_client *client,
 	text_input->input_panel_visible = false;
 
 	if (!wl_list_empty(&text_input->input_methods) &&
-	    text_input == text_input->manager->current_panel) {
-		text_input->manager->current_panel = NULL;
+	    text_input == text_input->manager->current_text_input)
 		wl_signal_emit(&ec->hide_input_panel_signal, ec);
-	}
 }
 
 static void
@@ -606,11 +614,13 @@ unbind_keyboard(struct wl_resource *resource)
 
 static void
 input_method_context_grab_key(struct weston_keyboard_grab *grab,
-			      uint32_t time, uint32_t key, uint32_t state_w)
+			      const struct timespec *time, uint32_t key,
+			      uint32_t state_w)
 {
 	struct weston_keyboard *keyboard = grab->keyboard;
 	struct wl_display *display;
 	uint32_t serial;
+	uint32_t msecs;
 
 	if (!keyboard->input_method_resource)
 		return;
@@ -618,8 +628,9 @@ input_method_context_grab_key(struct weston_keyboard_grab *grab,
 	display = wl_client_get_display(
 		wl_resource_get_client(keyboard->input_method_resource));
 	serial = wl_display_next_serial(display);
+	msecs = timespec_to_msec(time);
 	wl_keyboard_send_key(keyboard->input_method_resource,
-			     serial, time, key, state_w);
+			     serial, msecs, key, state_w);
 }
 
 static void
@@ -692,8 +703,11 @@ input_method_context_key(struct wl_client *client,
 	struct weston_seat *seat = context->input_method->seat;
 	struct weston_keyboard *keyboard = weston_seat_get_keyboard(seat);
 	struct weston_keyboard_grab *default_grab = &keyboard->default_grab;
+	struct timespec ts;
 
-	default_grab->interface->key(default_grab, time, key, state_w);
+	timespec_from_msec(&ts, time);
+
+	default_grab->interface->key(default_grab, &ts, key, state_w);
 }
 
 static void
@@ -930,11 +944,14 @@ static void launch_input_method(struct text_backend *text_backend);
 static void
 respawn_input_method_process(struct text_backend *text_backend)
 {
-	uint32_t time;
+	struct timespec time;
+	int64_t tdiff;
 
 	/* if input_method dies more than 5 times in 10 seconds, give up */
-	time = weston_compositor_get_time();
-	if (time - text_backend->input_method.deathstamp > 10000) {
+	weston_compositor_get_time(&time);
+	tdiff = timespec_sub_to_msec(&time,
+				     &text_backend->input_method.deathstamp);
+	if (tdiff > 10000) {
 		text_backend->input_method.deathstamp = time;
 		text_backend->input_method.deathcount = 0;
 	}
