@@ -303,8 +303,7 @@ static const struct zxdg_positioner_v6_interface weston_desktop_xdg_positioner_i
 };
 
 static void
-weston_desktop_xdg_surface_schedule_configure(struct weston_desktop_xdg_surface *surface,
-					      bool force);
+weston_desktop_xdg_surface_schedule_configure(struct weston_desktop_xdg_surface *surface);
 
 static void
 weston_desktop_xdg_toplevel_ensure_added(struct weston_desktop_xdg_toplevel *toplevel)
@@ -314,7 +313,7 @@ weston_desktop_xdg_toplevel_ensure_added(struct weston_desktop_xdg_toplevel *top
 
 	weston_desktop_api_surface_added(toplevel->base.desktop,
 					 toplevel->base.desktop_surface);
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, true);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 	toplevel->added = true;
 }
 
@@ -379,6 +378,9 @@ weston_desktop_xdg_toplevel_protocol_show_window_menu(struct wl_client *wl_clien
 		return;
 	}
 
+	if (seat == NULL)
+		return;
+
 	weston_desktop_api_show_window_menu(toplevel->base.desktop,
 					    dsurface, seat, x, y);
 }
@@ -403,6 +405,9 @@ weston_desktop_xdg_toplevel_protocol_move(struct wl_client *wl_client,
 		return;
 	}
 
+	if (seat == NULL)
+		return;
+
 	weston_desktop_api_move(toplevel->base.desktop, dsurface, seat, serial);
 }
 
@@ -429,6 +434,9 @@ weston_desktop_xdg_toplevel_protocol_resize(struct wl_client *wl_client,
 		return;
 	}
 
+	if (seat == NULL)
+		return;
+
 	weston_desktop_api_resize(toplevel->base.desktop,
 				  dsurface, seat, serial, surf_edges);
 }
@@ -584,7 +592,7 @@ weston_desktop_xdg_toplevel_set_maximized(struct weston_desktop_surface *dsurfac
 	struct weston_desktop_xdg_toplevel *toplevel = user_data;
 
 	toplevel->pending.state.maximized = maximized;
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, false);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 }
 
 static void
@@ -594,7 +602,7 @@ weston_desktop_xdg_toplevel_set_fullscreen(struct weston_desktop_surface *dsurfa
 	struct weston_desktop_xdg_toplevel *toplevel = user_data;
 
 	toplevel->pending.state.fullscreen = fullscreen;
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, false);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 }
 
 static void
@@ -604,7 +612,7 @@ weston_desktop_xdg_toplevel_set_resizing(struct weston_desktop_surface *dsurface
 	struct weston_desktop_xdg_toplevel *toplevel = user_data;
 
 	toplevel->pending.state.resizing = resizing;
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, false);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 }
 
 static void
@@ -614,7 +622,7 @@ weston_desktop_xdg_toplevel_set_activated(struct weston_desktop_surface *dsurfac
 	struct weston_desktop_xdg_toplevel *toplevel = user_data;
 
 	toplevel->pending.state.activated = activated;
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, false);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 }
 
 static void
@@ -627,7 +635,7 @@ weston_desktop_xdg_toplevel_set_size(struct weston_desktop_surface *dsurface,
 	toplevel->pending.size.width = width;
 	toplevel->pending.size.height = height;
 
-	weston_desktop_xdg_surface_schedule_configure(&toplevel->base, false);
+	weston_desktop_xdg_surface_schedule_configure(&toplevel->base);
 }
 
 static void
@@ -763,6 +771,12 @@ weston_desktop_xdg_popup_protocol_grab(struct wl_client *wl_client,
 	bool parent_is_toplevel =
 		popup->parent->role == WESTON_DESKTOP_XDG_SURFACE_ROLE_TOPLEVEL;
 
+	/* Check that if we have a valid wseat we also got a valid desktop seat */
+	if (wseat != NULL && seat == NULL) {
+		wl_client_post_no_memory(wl_client);
+		return;
+	}
+
 	if (popup->committed) {
 		wl_resource_post_error(popup->resource,
 				       ZXDG_POPUP_V6_ERROR_INVALID_GRAB,
@@ -770,6 +784,15 @@ weston_desktop_xdg_popup_protocol_grab(struct wl_client *wl_client,
 		return;
 	}
 
+	/* If seat is NULL then get_topmost_surface will return NULL. In
+	 * combination with setting parent_is_toplevel to TRUE here we will
+	 * avoid posting an error, and we will instead gracefully fail the
+	 * grab and dismiss the surface.
+	 * FIXME: this is a hack because currently we cannot check the topmost
+	 * parent with a destroyed weston_seat */
+	if (seat == NULL)
+		parent_is_toplevel = true;
+
 	topmost = weston_desktop_seat_popup_grab_get_topmost_surface(seat);
 	if ((topmost == NULL && !parent_is_toplevel) ||
 	    (topmost != NULL && topmost != popup->parent->desktop_surface)) {
@@ -807,7 +830,7 @@ static void
 weston_desktop_xdg_popup_committed(struct weston_desktop_xdg_popup *popup)
 {
 	if (!popup->committed)
-		weston_desktop_xdg_surface_schedule_configure(&popup->base, true);
+		weston_desktop_xdg_surface_schedule_configure(&popup->base);
 	popup->committed = true;
 	weston_desktop_xdg_popup_update_position(popup->base.desktop_surface,
 						 popup);
@@ -904,17 +927,39 @@ weston_desktop_xdg_surface_send_configure(void *user_data)
 static bool
 weston_desktop_xdg_toplevel_state_compare(struct weston_desktop_xdg_toplevel *toplevel)
 {
-	if (toplevel->pending.state.activated != toplevel->current.state.activated)
-		return false;
-	if (toplevel->pending.state.fullscreen != toplevel->current.state.fullscreen)
-		return false;
-	if (toplevel->pending.state.maximized != toplevel->current.state.maximized)
-		return false;
-	if (toplevel->pending.state.resizing != toplevel->current.state.resizing)
+	struct {
+		struct weston_desktop_xdg_toplevel_state state;
+		struct weston_size size;
+	} configured;
+
+	if (!toplevel->base.configured)
 		return false;
 
-	if (toplevel->base.surface->width == toplevel->pending.size.width &&
-	    toplevel->base.surface->height == toplevel->pending.size.height)
+	if (wl_list_empty(&toplevel->base.configure_list)) {
+		/* Last configure is actually the current state, just use it */
+		configured.state = toplevel->current.state;
+		configured.size.width = toplevel->base.surface->width;
+		configured.size.height = toplevel->base.surface->height;
+	} else {
+		struct weston_desktop_xdg_toplevel_configure *configure =
+			wl_container_of(toplevel->base.configure_list.prev,
+					configure, base.link);
+
+		configured.state = configure->state;
+		configured.size = configure->size;
+	}
+
+	if (toplevel->pending.state.activated != configured.state.activated)
+		return false;
+	if (toplevel->pending.state.fullscreen != configured.state.fullscreen)
+		return false;
+	if (toplevel->pending.state.maximized != configured.state.maximized)
+		return false;
+	if (toplevel->pending.state.resizing != configured.state.resizing)
+		return false;
+
+	if (toplevel->pending.size.width == configured.size.width &&
+	    toplevel->pending.size.height == configured.size.height)
 		return true;
 
 	if (toplevel->pending.size.width == 0 &&
@@ -925,20 +970,18 @@ weston_desktop_xdg_toplevel_state_compare(struct weston_desktop_xdg_toplevel *to
 }
 
 static void
-weston_desktop_xdg_surface_schedule_configure(struct weston_desktop_xdg_surface *surface,
-					      bool force)
+weston_desktop_xdg_surface_schedule_configure(struct weston_desktop_xdg_surface *surface)
 {
 	struct wl_display *display = weston_desktop_get_display(surface->desktop);
 	struct wl_event_loop *loop = wl_display_get_event_loop(display);
-	bool pending_same = !force;
+	bool pending_same = false;
 
 	switch (surface->role) {
 	case WESTON_DESKTOP_XDG_SURFACE_ROLE_NONE:
 		assert(0 && "not reached");
 		break;
 	case WESTON_DESKTOP_XDG_SURFACE_ROLE_TOPLEVEL:
-		pending_same = pending_same &&
-			weston_desktop_xdg_toplevel_state_compare((struct weston_desktop_xdg_toplevel *) surface);
+		pending_same = weston_desktop_xdg_toplevel_state_compare((struct weston_desktop_xdg_toplevel *) surface);
 		break;
 	case WESTON_DESKTOP_XDG_SURFACE_ROLE_POPUP:
 		break;
@@ -1327,6 +1370,7 @@ weston_desktop_xdg_shell_protocol_get_xdg_surface(struct wl_client *wl_client,
 
 	surface->desktop = weston_desktop_client_get_desktop(client);
 	surface->surface = wsurface;
+	wl_list_init(&surface->configure_list);
 
 	surface->desktop_surface =
 		weston_desktop_surface_create(surface->desktop, client,
@@ -1352,8 +1396,6 @@ weston_desktop_xdg_shell_protocol_get_xdg_surface(struct wl_client *wl_client,
 				       "xdg_surface must not have a buffer at creation");
 		return;
 	}
-
-	wl_list_init(&surface->configure_list);
 }
 
 static void
