@@ -55,6 +55,8 @@
 #include "shared/config-parser.h"
 #include "shared/helpers.h"
 #include "shared/image-loader.h"
+#include "shared/timespec-util.h"
+#include "shared/file-util.h"
 #include "gl-renderer.h"
 #include "weston-egl-ext.h"
 #include "pixman-renderer.h"
@@ -64,6 +66,12 @@
 
 #define DEFAULT_AXIS_STEP_DISTANCE 10
 
+#define WINDOW_MIN_WIDTH 128
+#define WINDOW_MIN_HEIGHT 128
+
+#define WINDOW_MAX_WIDTH 8192
+#define WINDOW_MAX_HEIGHT 8192
+
 struct x11_backend {
 	struct weston_backend	 base;
 	struct weston_compositor *compositor;
@@ -113,6 +121,7 @@ struct x11_output {
 
 	xcb_window_t		window;
 	struct weston_mode	mode;
+	struct weston_mode	native;
 	struct wl_event_source *finish_frame_timer;
 
 	xcb_gc_t		gc;
@@ -122,6 +131,8 @@ struct x11_output {
 	void		       *buf;
 	uint8_t			depth;
 	int32_t                 scale;
+	bool			resize_pending;
+	bool			window_resized;
 };
 
 struct window_delete_data {
@@ -771,6 +782,89 @@ x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
 	return 0;
 }
 
+static int
+x11_output_switch_mode(struct weston_output *base, struct weston_mode *mode)
+{
+	struct x11_backend *b;
+	struct x11_output *output;
+	static uint32_t values[2];
+	int ret;
+
+	if (base == NULL) {
+		weston_log("output is NULL.\n");
+		return -1;
+	}
+
+	if (mode == NULL) {
+		weston_log("mode is NULL.\n");
+		return -1;
+	}
+
+        b = to_x11_backend(base->compositor);
+        output = to_x11_output(base);
+
+        if (mode->width == output->mode.width &&
+	    mode->height == output->mode.height)
+	        return 0;
+
+        if (mode->width < WINDOW_MIN_WIDTH || mode->width > WINDOW_MAX_WIDTH)
+		return -1;
+
+	if (mode->height < WINDOW_MIN_HEIGHT || mode->height > WINDOW_MAX_HEIGHT)
+		return -1;
+
+	/* xcb_configure_window will create an event, and we could end up
+	   being called twice */
+	output->resize_pending = true;
+
+	/* window could've been resized by the user, so don't do it twice */
+	if (!output->window_resized) {
+		values[0] = mode->width;
+		values[1] = mode->height;
+		xcb_configure_window(b->conn, output->window, XCB_CONFIG_WINDOW_WIDTH |
+				     XCB_CONFIG_WINDOW_HEIGHT, values);
+	}
+
+	output->mode.width = mode->width;
+	output->mode.height = mode->height;
+
+	if (b->use_pixman) {
+		pixman_renderer_output_destroy(&output->base);
+		x11_output_deinit_shm(b, output);
+
+		if (x11_output_init_shm(b, output,
+		                        output->base.current_mode->width,
+		                        output->base.current_mode->height) < 0) {
+			weston_log("Failed to initialize SHM for the X11 output\n");
+			return -1;
+		}
+
+		if (pixman_renderer_output_create(&output->base) < 0) {
+			weston_log("Failed to create pixman renderer for output\n");
+			x11_output_deinit_shm(b, output);
+			return -1;
+		}
+	} else {
+		Window xid = (Window) output->window;
+
+		gl_renderer->output_destroy(&output->base);
+
+		ret = gl_renderer->output_window_create(&output->base,
+						        (EGLNativeWindowType) output->window,
+						        &xid,
+						        gl_renderer->opaque_attribs,
+						        NULL,
+						        0);
+		if (ret < 0)
+			return -1;
+	}
+
+	output->resize_pending = false;
+	output->window_resized = false;
+
+	return 0;
+}
+
 static int
 x11_output_disable(struct weston_output *base)
 {
@@ -801,7 +895,7 @@ x11_output_destroy(struct weston_output *base)
 	struct x11_output *output = to_x11_output(base);
 
 	x11_output_disable(&output->base);
-	weston_output_destroy(&output->base);
+	weston_output_release(&output->base);
 
 	free(output);
 }
@@ -818,6 +912,7 @@ x11_output_enable(struct weston_output *base)
 	xcb_screen_t *screen;
 	struct wm_normal_hints normal_hints;
 	struct wl_event_loop *loop;
+	char *icon_filename;
 
 	int ret;
 	uint32_t mask = XCB_CW_EVENT_MASK | XCB_CW_CURSOR;
@@ -863,14 +958,13 @@ x11_output_enable(struct weston_output *base)
 				    XCB_ATOM_ATOM, 32,
 				    ARRAY_LENGTH(atom_list), atom_list);
 	} else {
-		/* Don't resize me. */
 		memset(&normal_hints, 0, sizeof normal_hints);
 		normal_hints.flags =
 			WM_NORMAL_HINTS_MAX_SIZE | WM_NORMAL_HINTS_MIN_SIZE;
-		normal_hints.min_width = output->base.current_mode->width;
-		normal_hints.min_height = output->base.current_mode->height;
-		normal_hints.max_width = output->base.current_mode->width;
-		normal_hints.max_height = output->base.current_mode->height;
+		normal_hints.min_width = WINDOW_MIN_WIDTH;
+		normal_hints.min_height = WINDOW_MIN_HEIGHT;
+		normal_hints.max_width = WINDOW_MAX_WIDTH;
+		normal_hints.max_height = WINDOW_MAX_HEIGHT;
 		xcb_change_property(b->conn, XCB_PROP_MODE_REPLACE, output->window,
 				    b->atom.wm_normal_hints,
 				    b->atom.wm_size_hints, 32,
@@ -899,7 +993,9 @@ x11_output_enable(struct weston_output *base)
 			    b->atom.wm_class, b->atom.string, 8,
 			    sizeof class, class);
 
-	x11_output_set_icon(b, output, DATADIR "/weston/wayland.png");
+	icon_filename = file_name_with_datadir("wayland.png");
+	x11_output_set_icon(b, output, icon_filename);
+	free(icon_filename);
 
 	x11_output_set_wm_protocols(b, output);
 
@@ -944,7 +1040,7 @@ x11_output_enable(struct weston_output *base)
 	output->base.assign_planes = NULL;
 	output->base.set_backlight = NULL;
 	output->base.set_dpms = NULL;
-	output->base.switch_mode = NULL;
+	output->base.switch_mode = x11_output_switch_mode;
 
 	loop = wl_display_get_event_loop(b->compositor->wl_display);
 	output->finish_frame_timer =
@@ -977,13 +1073,13 @@ x11_output_set_size(struct weston_output *base, int width, int height)
 	/* Make sure we have scale set. */
 	assert(output->base.scale);
 
-	if (width < 1) {
+	if (width < WINDOW_MIN_WIDTH) {
 		weston_log("Invalid width \"%d\" for output %s\n",
 			   width, output->base.name);
 		return -1;
 	}
 
-	if (height < 1) {
+	if (height < WINDOW_MIN_HEIGHT) {
 		weston_log("Invalid height \"%d\" for output %s\n",
 			   height, output->base.name);
 		return -1;
@@ -998,14 +1094,17 @@ x11_output_set_size(struct weston_output *base, int width, int height)
 	output->mode.width = output_width;
 	output->mode.height = output_height;
 	output->mode.refresh = 60000;
+	output->native = output->mode;
 	output->scale = output->base.scale;
-	wl_list_init(&output->base.mode_list);
 	wl_list_insert(&output->base.mode_list, &output->mode.link);
 
 	output->base.current_mode = &output->mode;
 	output->base.make = "weston-X11";
 	output->base.model = "none";
 
+	output->base.native_mode = &output->native;
+	output->base.native_scale = output->base.scale;
+
 	output->base.mm_width = width * b->screen->width_in_millimeters /
 		b->screen->width_in_pixels;
 	output->base.mm_height = height * b->screen->height_in_millimeters /
@@ -1029,12 +1128,12 @@ x11_output_create(struct weston_compositor *compositor,
 		return -1;
 	}
 
-	output->base.name = strdup(name);
+	weston_output_init(&output->base, compositor, name);
+
 	output->base.destroy = x11_output_destroy;
 	output->base.disable = x11_output_disable;
 	output->base.enable = x11_output_enable;
 
-	weston_output_init(&output->base, compositor);
 	weston_compositor_add_pending_output(&output->base, compositor);
 
 	return 0;
@@ -1133,6 +1232,7 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 	struct x11_output *output;
 	struct weston_pointer_axis_event weston_event;
 	bool is_button_pressed = event->response_type == XCB_BUTTON_PRESS;
+	struct timespec time = { 0 };
 
 	assert(event->response_type == XCB_BUTTON_PRESS ||
 	       event->response_type == XCB_BUTTON_RELEASE);
@@ -1177,9 +1277,8 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 			weston_event.has_discrete = true;
 			weston_event.axis =
 				WL_POINTER_AXIS_VERTICAL_SCROLL;
-			notify_axis(&b->core_seat,
-				    weston_compositor_get_time(),
-				    &weston_event);
+			weston_compositor_get_time(&time);
+			notify_axis(&b->core_seat, &time, &weston_event);
 			notify_pointer_frame(&b->core_seat);
 		}
 		return;
@@ -1190,9 +1289,8 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 			weston_event.has_discrete = true;
 			weston_event.axis =
 				WL_POINTER_AXIS_VERTICAL_SCROLL;
-			notify_axis(&b->core_seat,
-				    weston_compositor_get_time(),
-				    &weston_event);
+			weston_compositor_get_time(&time);
+			notify_axis(&b->core_seat, &time, &weston_event);
 			notify_pointer_frame(&b->core_seat);
 		}
 		return;
@@ -1203,9 +1301,8 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 			weston_event.has_discrete = true;
 			weston_event.axis =
 				WL_POINTER_AXIS_HORIZONTAL_SCROLL;
-			notify_axis(&b->core_seat,
-				    weston_compositor_get_time(),
-				    &weston_event);
+			weston_compositor_get_time(&time);
+			notify_axis(&b->core_seat, &time, &weston_event);
 			notify_pointer_frame(&b->core_seat);
 		}
 		return;
@@ -1216,9 +1313,8 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 			weston_event.has_discrete = true;
 			weston_event.axis =
 				WL_POINTER_AXIS_HORIZONTAL_SCROLL;
-			notify_axis(&b->core_seat,
-				    weston_compositor_get_time(),
-				    &weston_event);
+			weston_compositor_get_time(&time);
+			notify_axis(&b->core_seat, &time, &weston_event);
 			notify_pointer_frame(&b->core_seat);
 		}
 		return;
@@ -1227,8 +1323,9 @@ x11_backend_deliver_button_event(struct x11_backend *b,
 		break;
 	}
 
-	notify_button(&b->core_seat,
-		      weston_compositor_get_time(), button,
+	weston_compositor_get_time(&time);
+
+	notify_button(&b->core_seat, &time, button,
 		      is_button_pressed ? WL_POINTER_BUTTON_STATE_PRESSED :
 					  WL_POINTER_BUTTON_STATE_RELEASED);
 	notify_pointer_frame(&b->core_seat);
@@ -1243,6 +1340,7 @@ x11_backend_deliver_motion_event(struct x11_backend *b,
 	struct weston_pointer_motion_event motion_event = { 0 };
 	xcb_motion_notify_event_t *motion_notify =
 			(xcb_motion_notify_event_t *) event;
+	struct timespec time;
 
 	if (!b->has_xkb)
 		update_xkb_state_from_core(b, motion_notify->state);
@@ -1261,8 +1359,8 @@ x11_backend_deliver_motion_event(struct x11_backend *b,
 		.dy = y - b->prev_y
 	};
 
-	notify_motion(&b->core_seat, weston_compositor_get_time(),
-		      &motion_event);
+	weston_compositor_get_time(&time);
+	notify_motion(&b->core_seat, &time, &motion_event);
 	notify_pointer_frame(&b->core_seat);
 
 	b->prev_x = x;
@@ -1320,12 +1418,14 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 	xcb_keymap_notify_event_t *keymap_notify;
 	xcb_focus_in_event_t *focus_in;
 	xcb_expose_event_t *expose;
+	xcb_configure_notify_event_t *configure;
 	xcb_atom_t atom;
 	xcb_window_t window;
 	uint32_t *k;
 	uint32_t i, set;
 	uint8_t response_type;
 	int count;
+	struct timespec time;
 
 	prev = NULL;
 	count = 0;
@@ -1352,8 +1452,9 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 				 * and fall through and handle the new
 				 * event below. */
 				update_xkb_state_from_core(b, key_release->state);
+				weston_compositor_get_time(&time);
 				notify_key(&b->core_seat,
-					   weston_compositor_get_time(),
+					   &time,
 					   key_release->detail - 8,
 					   WL_KEYBOARD_KEY_STATE_RELEASED,
 					   STATE_UPDATE_AUTOMATIC);
@@ -1396,8 +1497,9 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			key_press = (xcb_key_press_event_t *) event;
 			if (!b->has_xkb)
 				update_xkb_state_from_core(b, key_press->state);
+			weston_compositor_get_time(&time);
 			notify_key(&b->core_seat,
-				   weston_compositor_get_time(),
+				   &time,
 				   key_press->detail - 8,
 				   WL_KEYBOARD_KEY_STATE_PRESSED,
 				   b->has_xkb ? STATE_UPDATE_NONE :
@@ -1411,8 +1513,9 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 				break;
 			}
 			key_release = (xcb_key_press_event_t *) event;
+			weston_compositor_get_time(&time);
 			notify_key(&b->core_seat,
-				   weston_compositor_get_time(),
+				   &time,
 				   key_release->detail - 8,
 				   WL_KEYBOARD_KEY_STATE_RELEASED,
 				   STATE_UPDATE_NONE);
@@ -1471,6 +1574,31 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			}
 			break;
 
+		case XCB_CONFIGURE_NOTIFY:
+			configure = (struct xcb_configure_notify_event_t *) event;
+			struct x11_output *output =
+				x11_backend_find_output(b, configure->window);
+
+			if (!output || output->resize_pending)
+				break;
+
+			struct weston_mode mode = output->mode;
+
+			if (mode.width == configure->width &&
+			    mode.height == configure->height)
+				break;
+
+			output->window_resized = true;
+
+			mode.width = configure->width;
+			mode.height = configure->height;
+
+			if (weston_output_mode_set_native(&output->base,
+							  &mode, output->scale) < 0)
+				weston_log("Mode switch failed\n");
+
+			break;
+
 		case XCB_FOCUS_IN:
 			focus_in = (xcb_focus_in_event_t *) event;
 			if (focus_in->mode == XCB_NOTIFY_MODE_WHILE_GRABBED)
@@ -1518,8 +1646,9 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 	case XCB_KEY_RELEASE:
 		key_release = (xcb_key_press_event_t *) prev;
 		update_xkb_state_from_core(b, key_release->state);
+		weston_compositor_get_time(&time);
 		notify_key(&b->core_seat,
-			   weston_compositor_get_time(),
+			   &time,
 			   key_release->detail - 8,
 			   WL_KEYBOARD_KEY_STATE_RELEASED,
 			   STATE_UPDATE_AUTOMATIC);
@@ -1613,11 +1742,6 @@ x11_backend_get_wm_info(struct x11_backend *c)
 	free(reply);
 }
 
-static void
-x11_restore(struct weston_compositor *ec)
-{
-}
-
 static void
 x11_destroy(struct weston_compositor *ec)
 {
@@ -1670,6 +1794,8 @@ x11_backend_create(struct weston_compositor *compositor,
 	b->fullscreen = config->fullscreen;
 	b->no_input = config->no_input;
 
+	compositor->backend = &b->base;
+
 	if (weston_compositor_set_presentation_clock_software(compositor) < 0)
 		goto err_free;
 
@@ -1708,7 +1834,6 @@ x11_backend_create(struct weston_compositor *compositor,
 	weston_log("Using %s renderer\n", config->use_pixman ? "pixman" : "gl");
 
 	b->base.destroy = x11_destroy;
-	b->base.restore = x11_restore;
 
 	if (x11_input_create(b, config->no_input) < 0) {
 		weston_log("Failed to create X11 input\n");
@@ -1729,8 +1854,6 @@ x11_backend_create(struct weston_compositor *compositor,
 				   "support failed.\n");
 	}
 
-	compositor->backend = &b->base;
-
 	ret = weston_plugin_api_register(compositor, WESTON_WINDOWED_OUTPUT_API_NAME,
 					 &api, sizeof(api));
 
