@@ -86,16 +86,9 @@ weston_launcher_activate_vt(struct weston_launcher *launcher, int vt)
 	return launcher->iface->activate_vt(launcher, vt);
 }
 
-WL_EXPORT void
-weston_launcher_restore(struct weston_launcher *launcher)
-{
-	launcher->iface->restore(launcher);
-}
-
-
 static void
 switch_vt_binding(struct weston_keyboard *keyboard,
-		  uint32_t time, uint32_t key, void *data)
+		  const struct timespec *time, uint32_t key, void *data)
 {
 	struct weston_compositor *compositor = data;
 	struct weston_launcher *launcher = compositor->launcher;
