@@ -2,23 +2,26 @@
  * Copyright © 2008 Kristian Høgsberg
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -32,10 +35,13 @@
 #include <cairo.h>
 #include "cairo-util.h"
 
+#include "shared/helpers.h"
 #include "image-loader.h"
 #include "config-parser.h"
 
-#define ARRAY_LENGTH(a) (sizeof (a) / sizeof (a)[0])
+#ifdef HAVE_PANGO
+#include <pango/pangocairo.h>
+#endif
 
 void
 surface_flush_device(cairo_surface_t *surface)
@@ -137,13 +143,14 @@ blur_surface(cairo_surface_t *surface, int margin)
 }
 
 void
-tile_mask(cairo_t *cr, cairo_surface_t *surface,
-	  int x, int y, int width, int height, int margin, int top_margin)
+render_shadow(cairo_t *cr, cairo_surface_t *surface,
+	      int x, int y, int width, int height, int margin, int top_margin)
 {
 	cairo_pattern_t *pattern;
 	cairo_matrix_t matrix;
 	int i, fx, fy, shadow_height, shadow_width;
 
+	cairo_set_source_rgba(cr, 0, 0, 0, 0.45);
 	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
 	pattern = cairo_pattern_create_for_surface (surface);
 	cairo_pattern_set_filter(pattern, CAIRO_FILTER_NEAREST);
@@ -448,16 +455,45 @@ theme_destroy(struct theme *t)
 	free(t);
 }
 
+#ifdef HAVE_PANGO
+static PangoLayout *
+create_layout(cairo_t *cr, const char *title)
+{
+	PangoLayout *layout;
+	PangoFontDescription *desc;
+
+	layout = pango_cairo_create_layout(cr);
+	pango_layout_set_text(layout, title, -1);
+	desc = pango_font_description_from_string("Sans Bold 10");
+	pango_layout_set_font_description(layout, desc);
+	pango_font_description_free(desc);
+	pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_END);
+	pango_layout_set_alignment(layout, PANGO_ALIGN_LEFT);
+	pango_layout_set_auto_dir (layout, FALSE);
+	pango_layout_set_single_paragraph_mode (layout, TRUE);
+	pango_layout_set_width (layout, -1);
+
+	return layout;
+}
+#endif
+
+#ifdef HAVE_PANGO
+#define SHOW_TEXT(cr) \
+	pango_cairo_show_layout(cr, title_layout)
+#else
+#define SHOW_TEXT(cr) \
+	cairo_show_text(cr, title)
+#endif
+
 void
 theme_render_frame(struct theme *t,
 		   cairo_t *cr, int width, int height,
-		   const char *title, struct wl_list *buttons,
-		   uint32_t flags)
+		   const char *title, cairo_rectangle_int_t *title_rect,
+		   struct wl_list *buttons, uint32_t flags)
 {
-	cairo_text_extents_t extents;
-	cairo_font_extents_t font_extents;
 	cairo_surface_t *source;
 	int x, y, margin, top_margin;
+	int text_width, text_height;
 
 	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 	cairo_set_source_rgba(cr, 0, 0, 0, 0);
@@ -466,10 +502,9 @@ theme_render_frame(struct theme *t,
 	if (flags & THEME_FRAME_MAXIMIZED)
 		margin = 0;
 	else {
-		cairo_set_source_rgba(cr, 0, 0, 0, 0.45);
-		tile_mask(cr, t->shadow,
-			  2, 2, width + 8, height + 8,
-			  64, 64);
+		render_shadow(cr, t->shadow,
+			      2, 2, width + 8, height + 8,
+			      64, 64);
 		margin = t->margin;
 	}
 
@@ -489,35 +524,56 @@ theme_render_frame(struct theme *t,
 		    t->width, top_margin);
 
 	if (title || !wl_list_empty(buttons)) {
-		cairo_rectangle (cr, margin + t->width, margin,
-				 width - (margin + t->width) * 2,
-				 t->titlebar_height - t->width);
-		cairo_clip(cr);
 
+		cairo_rectangle (cr, title_rect->x, title_rect->y,
+				 title_rect->width, title_rect->height);
+		cairo_clip(cr);
 		cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+
+#ifdef HAVE_PANGO
+		PangoLayout *title_layout;
+		PangoRectangle logical;
+
+		title_layout = create_layout(cr, title);
+
+		pango_layout_get_pixel_extents (title_layout, NULL, &logical);
+		text_width = MIN(title_rect->width, logical.width);
+		text_height = logical.height;
+		if (text_width < logical.width)
+		  pango_layout_set_width (title_layout, text_width * PANGO_SCALE);
+		
+#else
+		cairo_text_extents_t extents;
+		cairo_font_extents_t font_extents;
+
 		cairo_select_font_face(cr, "sans",
 				       CAIRO_FONT_SLANT_NORMAL,
 				       CAIRO_FONT_WEIGHT_BOLD);
 		cairo_set_font_size(cr, 14);
 		cairo_text_extents(cr, title, &extents);
 		cairo_font_extents (cr, &font_extents);
-		x = (width - extents.width) / 2;
-		y = margin +
-			(t->titlebar_height -
-			 font_extents.ascent - font_extents.descent) / 2 +
-			font_extents.ascent;
+		text_width = extents.width;
+		text_height = font_extents.descent - font_extents.ascent;
+#endif
+
+		x = (width - text_width) / 2;
+		y = margin + (t->titlebar_height - text_height) / 2;
+		if (x < title_rect->x)
+			x = title_rect->x;
+		else if (x + text_width > (title_rect->x + title_rect->width))
+			x = (title_rect->x + title_rect->width) - text_width;
 
 		if (flags & THEME_FRAME_ACTIVE) {
 			cairo_move_to(cr, x + 1, y  + 1);
 			cairo_set_source_rgb(cr, 1, 1, 1);
-			cairo_show_text(cr, title);
+			SHOW_TEXT(cr);
 			cairo_move_to(cr, x, y);
 			cairo_set_source_rgb(cr, 0, 0, 0);
-			cairo_show_text(cr, title);
+			SHOW_TEXT(cr);
 		} else {
 			cairo_move_to(cr, x, y);
 			cairo_set_source_rgb(cr, 0.4, 0.4, 0.4);
-			cairo_show_text(cr, title);
+			SHOW_TEXT(cr);
 		}
 	}
 }
